@@ -13,6 +13,7 @@ from anchorpy import Program, Context, Idl
 from driftpy.addresses import (
     get_user_account_public_key_and_nonce,
     get_user_orders_account_public_key_and_nonce,
+    get_order_state_account_public_key_and_nonce,
 )
 from driftpy.types import (
     PositionDirection,
@@ -27,6 +28,9 @@ from driftpy.types import (
     ExtendedCurveHistoryAccount,
     User,
     UserPositions,
+    OrderState,
+    OrderHistoryAccount,
+    OrderParams,
 )
 
 from driftpy.program import load_program
@@ -48,6 +52,8 @@ class ClearingHousePDAs:
     funding_rate_history: PublicKey
     liquidation_history: PublicKey
     curve_history: PublicKey
+    extended_curve_history: PublicKey
+    order_state: PublicKey
 
 
 def get_clearing_house_state_account_public_key_and_nonce(
@@ -133,6 +139,23 @@ class ClearingHouse:
         res = await self.program.account["CurveHistory"].fetch(self.pdas.curve_history)
         return cast(ExtendedCurveHistoryAccount, res)
 
+    async def get_extended_curve_history_account(self) -> ExtendedCurveHistoryAccount:
+        res = await self.program.account["ExtendedCurveHistory"].fetch(
+            self.pdas.extended_curve_history
+        )
+        return cast(ExtendedCurveHistoryAccount, res)
+
+    async def get_orders_state_account(self) -> OrderState:
+        res = await self.program.account["OrderState"].fetch(self.pdas.order_state)
+        return cast(OrderState, res)
+
+    async def get_orders_history_account(self) -> OrderHistoryAccount:
+        order_state = await self.get_orders_state_account()
+        res = await self.program.account["OrderHistory"].fetch(
+            order_state.order_history
+        )
+        return cast(OrderHistoryAccount, res)
+
     @staticmethod
     def _get_state_pubkey(program: Program) -> PublicKey:
         return PublicKey.find_program_address([b"clearing_house"], program.program_id)[
@@ -160,6 +183,8 @@ class ClearingHouse:
             funding_rate_history=state.funding_rate_history,
             liquidation_history=state.liquidation_history,
             curve_history=state.curve_history,
+            extended_curve_history=state.extended_curve_history,
+            order_state=state.order_state,
         )
         return cls(program, pdas)
 
@@ -185,6 +210,8 @@ class ClearingHouse:
             funding_rate_history=state.funding_rate_history,
             liquidation_history=state.liquidation_history,
             curve_history=state.curve_history,
+            order_state=state.order_state,
+            extended_curve_history=state.extended_curve_history,
         )
         return cls(program, pdas)
 
@@ -308,6 +335,10 @@ class ClearingHouse:
             self.program.program_id, self.program.provider.wallet.public_key
         )[0]
 
+    def get_order_state_public_key(self) -> PublicKey:
+        """Get the address for the Clearing House Order State account."""
+        return get_order_state_account_public_key_and_nonce(self.program.program_id)[0]
+
     async def get_withdraw_collateral_ix(
         self,
         amount: int,
@@ -447,6 +478,230 @@ class ClearingHouse:
             ),
         )
 
+    async def place_order(
+        self,
+        order_params: OrderParams,
+        discount_token: Optional[PublicKey] = None,
+        referrer: Optional[PublicKey] = None,
+    ) -> TransactionSignature:
+        ix = await self.get_place_order_ix(order_params, discount_token, referrer)
+        tx = Transaction().add(ix)
+        return await self.program.provider.send(tx)
+
+    async def get_place_order_ix(
+        self,
+        order_params: OrderParams,
+        discount_token: Optional[PublicKey] = None,
+        referrer: Optional[PublicKey] = None,
+        user_account: Optional[User] = None,
+        markets_account: Optional[MarketsAccount] = None,
+        state_account: Optional[StateAccount] = None,
+        orders_state_account: Optional[OrderState] = None,
+    ) -> TransactionInstruction:
+        user_account_public_key = self.get_user_account_public_key()
+        user_account_to_use = (
+            await self.get_user_account() if user_account is None else user_account
+        )
+        optional_accounts = {
+            "discount_token": False,
+            "referrer": False,
+        }
+        remaining_accounts = []
+        if discount_token:
+            optional_accounts["discount_token"] = True
+            remaining_accounts.append(
+                AccountMeta(
+                    pubkey=discount_token,
+                    is_writable=False,
+                    is_signer=False,
+                )
+            )
+        if referrer:
+            optional_accounts["referrer"] = True
+            remaining_accounts.append(
+                AccountMeta(
+                    pubkey=referrer,
+                    is_writable=True,
+                    is_signer=False,
+                )
+            )
+
+        markets_account_to_use = (
+            await self.get_markets_account()
+            if markets_account is None
+            else markets_account
+        )
+        price_oracle = markets_account_to_use.markets[
+            order_params.market_index
+        ].amm.oracle
+
+        state = (
+            await self.get_state_account() if state_account is None else state_account
+        )
+
+        orders_state = (
+            await self.get_orders_state_account()
+            if orders_state_account is None
+            else orders_state_account
+        )
+
+        return self.program.instruction["place_order"](
+            order_params,
+            ctx=Context(
+                accounts={
+                    "state": self.pdas.state,
+                    "user": user_account_public_key,
+                    "authority": self.program.provider.wallet.public_key,
+                    "markets": state.markets,
+                    "user_orders": self.get_user_orders_public_key(),
+                    "user_positions": user_account_to_use.positions,
+                    "funding_payment_history": state.funding_payment_history,
+                    "funding_rate_history": state.funding_rate_history,
+                    "order_state": self.get_order_state_public_key(),
+                    "order_history": orders_state.order_history,
+                    "oracle": price_oracle,
+                },
+                remaining_accounts=remaining_accounts,
+            ),
+        )
+
+    async def place_and_fill_order(
+        self,
+        order_params: OrderParams,
+        discount_token: Optional[PublicKey] = None,
+        referrer: Optional[PublicKey] = None,
+    ) -> TransactionSignature:
+        ix = await self.get_place_and_fill_order_ix(
+            order_params, discount_token, referrer
+        )
+        tx = Transaction().add(ix)
+        return await self.program.provider.send(tx)
+
+    async def get_place_and_fill_order_ix(
+        self,
+        order_params: OrderParams,
+        discount_token: Optional[PublicKey] = None,
+        referrer: Optional[PublicKey] = None,
+        user_account: Optional[User] = None,
+        markets_account: Optional[MarketsAccount] = None,
+        state_account: Optional[StateAccount] = None,
+        orders_state_account: Optional[OrderState] = None,
+    ) -> TransactionInstruction:
+        user_account_public_key = self.get_user_account_public_key()
+        user_account_to_use = (
+            await self.get_user_account() if user_account is None else user_account
+        )
+        optional_accounts = {
+            "discount_token": False,
+            "referrer": False,
+        }
+        remaining_accounts = []
+        if discount_token:
+            optional_accounts["discount_token"] = True
+            remaining_accounts.append(
+                AccountMeta(
+                    pubkey=discount_token,
+                    is_writable=False,
+                    is_signer=False,
+                )
+            )
+        if referrer:
+            optional_accounts["referrer"] = True
+            remaining_accounts.append(
+                AccountMeta(
+                    pubkey=referrer,
+                    is_writable=True,
+                    is_signer=False,
+                )
+            )
+
+        markets_account_to_use = (
+            await self.get_markets_account()
+            if markets_account is None
+            else markets_account
+        )
+        price_oracle = markets_account_to_use.markets[
+            order_params.market_index
+        ].amm.oracle
+
+        state = (
+            await self.get_state_account() if state_account is None else state_account
+        )
+
+        orders_state = (
+            await self.get_orders_state_account()
+            if orders_state_account is None
+            else orders_state_account
+        )
+
+        return self.program.instruction["place_and_fill_order"](
+            order_params,
+            ctx=Context(
+                accounts={
+                    "state": self.pdas.state,
+                    "user": user_account_public_key,
+                    "authority": self.program.provider.wallet.public_key,
+                    "markets": state.markets,
+                    "user_orders": self.get_user_orders_public_key(),
+                    "user_positions": user_account_to_use.positions,
+                    "trade_history": state.trade_history,
+                    "funding_payment_history": state.funding_payment_history,
+                    "funding_rate_history": state.funding_rate_history,
+                    "order_state": self.get_order_state_public_key(),
+                    "order_history": orders_state.order_history,
+                    "extended_curve_history": state.extended_curve_history,
+                    "oracle": price_oracle,
+                },
+                remaining_accounts=remaining_accounts,
+            ),
+        )
+
+    async def cancel_order(
+        self,
+        order_id: int,
+    ) -> TransactionSignature:
+        ix = await self.get_cancel_order_ix(order_id)
+        tx = Transaction().add(ix)
+        return await self.program.provider.send(tx)
+
+    async def get_cancel_order_ix(
+        self,
+        order_id: int,
+        user_account: Optional[User] = None,
+        state_account: Optional[StateAccount] = None,
+        orders_state_account: Optional[OrderState] = None,
+    ) -> TransactionInstruction:
+        user_account_public_key = self.get_user_account_public_key()
+        user_account_to_use = (
+            await self.get_user_account() if user_account is None else user_account
+        )
+        state = (
+            await self.get_state_account() if state_account is None else state_account
+        )
+        orders_state = (
+            await self.get_orders_state_account()
+            if orders_state_account is None
+            else orders_state_account
+        )
+
+        return self.program.instruction["cancel_order"](
+            order_id,
+            ctx=Context(
+                accounts={
+                    "state": self.pdas.state,
+                    "user": user_account_public_key,
+                    "authority": self.program.provider.wallet.public_key,
+                    "markets": state.markets,
+                    "user_orders": self.get_user_orders_public_key(),
+                    "user_positions": user_account_to_use.positions,
+                    "funding_payment_history": state.funding_payment_history,
+                    "funding_rate_history": state.funding_rate_history,
+                    "order_state": self.get_order_state_public_key(),
+                    "order_history": orders_state.order_history,
+                },
+            ),
+        )
+
     async def get_user_account(self) -> User:
         user_account_pubkey = self.get_user_account_public_key()
         return cast(User, await self.program.account["User"].fetch(user_account_pubkey))
